package jp.co.atmarkit.user.web.logic {
import flash.data.SQLConnection;
import flash.data.SQLResult;
import flash.data.SQLStatement;
import flash.errors.IOError;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.events.SQLErrorEvent;
import flash.events.SQLEvent;
import flash.filesystem.File;
import flash.filesystem.FileMode;
import flash.filesystem.FileStream;

import jp.co.atmarkit.user.cnst.MessageConsts;
import jp.co.atmarkit.user.dto.UserRequestDto;
import jp.co.atmarkit.user.dto.UserResponseDto;
import jp.co.atmarkit.user.entity.User;
import jp.co.atmarkit.user.logic.Logic;
import jp.co.atmarkit.user.web.UserForm;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.controls.advancedDataGridClasses.AdvancedDataGridColumn;
import mx.events.FlexEvent;
import mx.events.ListEvent;
import mx.managers.CursorManager;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
    
/**
 * ユーザー管理アプリケーションのロジッククラスです。
 */
public class UserLogic extends Logic {
//--------------------------------------
//  Variables
//--------------------------------------

    [Bindable]
    /** ユーザーのリストです。 */
    public var userList:ArrayCollection;

    /** SQLiteからSELECTしてきたユーザーのリストです。 */
    public var selectedUserList:ArrayCollection;

    [Bindable]
    /** ヒット件数です。 */
    public var hitNum:String;

    /** 選択された性別絞り込み条件です。 */
    private var selectedSex:String;

    /** 選択された血液型絞り込み条件です。 */
    private var selectedBloodType:String;

    /** 選択された都道府県絞り込み条件です。 */
    private var selectedPre:String;

    /** 入力されたEメール絞り込み条件です。 */
    private var selectedEmail:String;

//--------------------------------------
//  Variables:DB Related
//--------------------------------------

    /** SQLConnectionオブジェクトです。 */
    private var sqlConnection:SQLConnection;
    
    /** SQLStatementオブジェクトです。 */
    private var sqlStatement:SQLStatement;
    
    /** INSERT処理のループインデックスです。 */
    private var loopIndex:uint;
    
    /** INSERT処理のループインデックス限界値です。 */
    private var loopIndexLimit:uint;
    
//--------------------------------------
//  Constructor
//--------------------------------------
    /**
     * コンストラクタです。
     */
    public function UserLogic() {
    }

//--------------------------------------
//  Initialization
//--------------------------------------
    /**
     * onCreationCompleteHandler
     * 
     * @param event FlexEvent
     */
    override protected function onCreationCompleteHandler(event:FlexEvent):void {
    }

//--------------------------------------
//  UI Event Handler
//--------------------------------------

    /**
     * getButton押下処理です。
     * 
     * @param event MouseEventオブジェクトです。
     */
    public function getButtonClickHandler(event:MouseEvent):void {
        // DTOの準備
        var requestDto:UserRequestDto = new UserRequestDto();
        
        // 特に送信するデータが無いので空のまま処理を呼び出し
        
        // リモートメソッドを呼び出す
        view.srvS2BlazeDS.getUser(requestDto);
        
        // 利用不可処理
        view.controlBar.enabled = false;
    }

    /**
     * sqliteButton押下処理です。
     * 
     * @param event MouseEventオブジェクトです。
     */
    public function sqliteButtonClickHandler(event:MouseEvent):void {
        // 利用可処理
        view.controlBar.enabled = false;
        
        // ビジーカーソル表示
        CursorManager.setBusyCursor();

         // データベースを開きます。存在しない場合は新規作成されます
        var dbFile:File = new File(File.applicationDirectory.nativePath + "/data/atmarkit.db");

        // SQLConnectionを新規作成します。
        sqlConnection = new SQLConnection();

        // イベント登録
        sqlConnection.addEventListener(SQLEvent.OPEN, sqlConnectionOpenHandler);
        sqlConnection.addEventListener(SQLErrorEvent.ERROR, sqlConnectionErrorHandler);
        
        // openAsync()によってデータベースを開きます。
        // 非同期モードで開くことにより、長時間のクエリに対してアプリが停止しないようにします
        // opneで開いた場合、メインスレッドが利用されるために、長時間のクエリではアプリが固まってしまいます
        sqlConnection.openAsync(dbFile);
    }

    /**
     * xmlButton押下処理です。
     * 
     * @param event MouseEventオブジェクトです。
     */
    public function xmlButtonClickHandler(event:MouseEvent):void {
        // ビジーカーソル表示
        CursorManager.setBusyCursor();
        
        // SQLiteデータベースからデータを取得します。
        selectRecord();
    }

//--------------------------------------
//  Server Connection Event Handler
//--------------------------------------
    
    /** 
     * ユーザーをサーバーから取得する：結果イベントです。
     * 
     * @param event ResultEventオブジェクトです。
     */
    public function getUserResultHandler(event:ResultEvent):void {
        // DTOを取得
        var responseDto:UserResponseDto = event.result as UserResponseDto;
        
        // 結果を取得
        userList = responseDto.userList;
        
        // 共通後処理
        initDisplay();
    }

    /** 
     * ユーザーをサーバーから取得する：失敗イベントです。
     * 
     * @param event FaultEventオブジェクトです。
     */
    public function getUserFaultHandler(event:FaultEvent):void {
        trace(event.fault.message);
        Alert.show(MessageConsts.ERROR_SERVER_ERROR);
        
        // 共通後処理
        initDisplay();
    }
    
    /**
     * サーバー処理後の共通後処理です。
     */
    private function initDisplay():void {
        // 利用可処理
        view.controlBar.enabled = true;
        view.getButton.enabled = false;
        view.sqliteButton.enabled = true;
        view.xmlButton.enabled = false;
        
        // フィルター条件を初期状態にする
        view.sexCombo.selectedIndex = 0;
        view.bloodCombo.selectedIndex = 0;
        view.preCombo.selectedIndex = 0;
        view.emailText.text = "";
        
        // 表示件数更新
        updateHitNum();
    }

//--------------------------------------
//  DB Event Handlers
//--------------------------------------

    /**
     * データベースとの接続結果ハンドラです。
     * 
     * @param event SQLEvent
     */
    private function sqlConnectionOpenHandler(event:SQLEvent):void {
        trace(MessageConsts.MESSAGE_SQLITE_CONNECTION_SUCCESS);

        createTable();
    }

    /**
     * CREATE処理です。
     */
    private function createTable():void {
        // テーブルを作成します
        sqlStatement = new SQLStatement();
        sqlStatement.sqlConnection = sqlConnection;

        // テーブルの作成
        // エンティティとのORマップのため、カラム名をASの命名規則に準ずるようにします
        // （SELECT処理時に重要です）
        sqlStatement.text =
            "CREATE TABLE IF NOT EXISTS USER (" +
            "id INTEGER PRIMARY KEY, " +
            "name TEXT, " +
            "ruby TEXT, " +
            "mail TEXT, " +
            "sex TEXT, " +
            "age TEXT, " +
            "birthday TEXT, " +
            "married TEXT, " +
            "bloodtype TEXT, " +
            "prefecture TEXT, " +
            "prefectureCode TEXT, " +
            "phone TEXT, " +
            "keitai TEXT, " +
            "carrier TEXT, " +
            "curry TEXT" +
            ")";

        // イベント登録
        sqlStatement.addEventListener(SQLEvent.RESULT, sqlStatementCreateResultHandler);
        sqlStatement.addEventListener(SQLErrorEvent.ERROR, sqlStatementErrorHandler);

        // 実行
        sqlStatement.execute();
    }
    
    /**
     * CREATE処理結果ハンドラです。
     * 
     * @param event SQLEvent
     */
    private function sqlStatementCreateResultHandler(event:SQLEvent):void {
        trace(MessageConsts.MESSAGE_SQLITE_TABLE_CREATION_SUCCESS);
        
        deleteRecord();
    }

    /**
     * DELETE処理です。
     */
    public function deleteRecord():void {
        // 既存データが存在していた場合、いったん削除します。

        sqlStatement = new SQLStatement();
        sqlStatement.sqlConnection =  sqlConnection;
        sqlStatement.text = "DELETE FROM USER";

        //イベント登録
        sqlStatement.addEventListener(SQLEvent.RESULT, sqlStatementDeleteResultHandler);
        sqlStatement.addEventListener(SQLErrorEvent.ERROR, sqlStatementErrorHandler);

        //実行
        sqlStatement.execute();
    }

    /**
     * DELETE処理結果ハンドラです。
     * 
     * @param event SQLEvent
     */
    private function sqlStatementDeleteResultHandler(event:SQLEvent):void {
        trace(MessageConsts.MESSAGE_SQLITE_DATA_DELETION_SUCCESS);
        
        insertRecord();
    }

    /**
     * INSERT処理です。
     */
    private function insertRecord():void {
        // インデックスをメソッド間で共有できるように保存
        loopIndex = 0;
        loopIndexLimit = userList.length;

        // トランザクション開始
        sqlConnection.begin();

        sqlStatement = new SQLStatement();
        sqlStatement.sqlConnection =  sqlConnection;

        // イベント登録
        sqlStatement.addEventListener(SQLEvent.RESULT, sqlStatementInsertResultHandler);
        sqlStatement.addEventListener(SQLErrorEvent.ERROR, sqlStatementErrorHandler);
        
        // 1件目のINSERT処理に移ります。
        insert();
    }
    
    /**
     * 個別INSERT処理です。
     */
    private function insert():void {
        // traceが記述されているとパフォーマンスが大幅に下がるので要らなくなったら削除しましょう。
        // 普通はロギングのフレームワークを作って対応します。
        //trace("insert()", loopIndex);
        
        // Insertするデータ
        var user:User = User(userList.getItemAt(loopIndex));

        // SQLです。
        // パラメータはコロン（:）かアットマーク（@）で開始します。クエスチョンマーク（?）で指定した場合は、下記parametersではインデックスで値を指定します。
        // パラメータ指定を行うことによりSQLインジェクション対策が行われます。
        var mySQL:String = "INSERT INTO USER(name, ruby, mail, sex, age, birthday, married, bloodtype, prefecture, prefectureCode, phone, keitai, carrier, curry) " +
            "VALUES(:name, :ruby, :mail, :sex, :age, :birthday, :married, :bloodtype, :prefecture, :prefectureCode, :phone, :keitai, :carrier, :curry)"
            ;
        
        sqlStatement.parameters[":name"] = user.name;
        sqlStatement.parameters[":ruby"] = user.ruby;
        sqlStatement.parameters[":mail"] = user.mail;
        sqlStatement.parameters[":sex"] = user.sex;
        sqlStatement.parameters[":age"] = user.age;
        sqlStatement.parameters[":birthday"] = user.birthday;
        sqlStatement.parameters[":married"] = user.married;
        sqlStatement.parameters[":bloodtype"] = user.bloodtype;
        sqlStatement.parameters[":prefecture"] = user.prefecture;
        sqlStatement.parameters[":prefectureCode"] = user.prefectureCode;
        sqlStatement.parameters[":phone"] = user.phone;
        sqlStatement.parameters[":keitai"] = user.keitai;
        sqlStatement.parameters[":carrier"] = user.carrier;
        sqlStatement.parameters[":curry"] = user.curry;

        sqlStatement.text = mySQL;

        //実行
        sqlStatement.execute();
    }
    
    /**
     * INSERT処理結果ハンドラです。
     * 
     * @param event SQLEvent
     */
    private function sqlStatementInsertResultHandler(event:SQLEvent):void {
        loopIndex++;

        // 処理状況により引き続きInsertするかどうかをチェック
        if(loopIndex < loopIndexLimit) {
            insert();
        } else {
            Alert.show(MessageConsts.MESSAGE_SQLITE_DATA_INSERTION_SUCCESS);
    
            // [loopIndexLimit]件分の処理を一気にコミット
            sqlConnection.commit();
            
            // 後処理
            initDisplay2();
        }
    }

    /**
     * INSERT処理後の後処理です。
     */
    private function initDisplay2():void {
        // ビジーカーソルを削除する
        CursorManager.removeBusyCursor();

        // 利用可処理
        view.controlBar.enabled = true;
        view.getButton.enabled = false;
        view.sqliteButton.enabled = false;
        view.xmlButton.enabled = true;
        
        // フィルター条件を初期状態にする
        view.sexCombo.selectedIndex = 0;
        view.bloodCombo.selectedIndex = 0;
        view.preCombo.selectedIndex = 0;
        view.emailText.text = "";
        
        // 表示件数更新
        updateHitNum();
    }

    /**
     * SELECT処理です。
     */
    private function selectRecord():void {
        sqlStatement = new SQLStatement();
        sqlStatement.sqlConnection =  sqlConnection;
        sqlStatement.text =
            "SELECT id, name, ruby, mail, sex, age, birthday, married, bloodtype, prefecture, prefectureCode, phone, keitai, carrier, curry FROM USER";
        
        // ここ重要！！！！！
        // ステートメントの実行結果として返された各行で使用されているクラス（データ型）を示します。（ヘルプより）
        sqlStatement.itemClass = User;

        //イベント登録
        sqlStatement.addEventListener(SQLEvent.RESULT, selectRecordResultHandler);
        sqlStatement.addEventListener(SQLErrorEvent.ERROR, sqlStatementErrorHandler);

        //実行
        sqlStatement.execute();
    }

    /**
     * SELECT処理結果ハンドラです。
     *
     * @param event SQLEvent
     */
    private function selectRecordResultHandler(event:SQLEvent):void {
        var result:SQLResult = sqlStatement.getResult();

        // ステートメントがデータを返さない場合
        if (result.data == null) {
            return;
        }

        // 取得データを保存
        selectedUserList = new ArrayCollection(result.data);
        
        writeRecord();
    }

    /**
     * データベースとの接続失敗ハンドラです。
     *
     * @param event SQLErrorEventオブジェクトです。
     */
    private function sqlConnectionErrorHandler(event:SQLErrorEvent):void {
        trace(MessageConsts.ERROR_SQLITE_CONNECTION_ERROR);
        trace(event.error.message);
        
        // ビジーカーソルを削除します。
        CursorManager.removeBusyCursor();
    }

    /**
     * SQLステートメント実行失敗ハンドラです。
     *
     * @param event SQLErrorEventオブジェクトです。
     */
    private function sqlStatementErrorHandler(event:SQLErrorEvent):void {
        trace(MessageConsts.ERROR_SQLITE_STATEMENT_ERROR);
        trace(event.error.message);

        // ビジーカーソルを削除します。
        CursorManager.removeBusyCursor();
    }

//--------------------------------------
//  File Operation
//--------------------------------------

    /**
     * XML書き出し処理です。
     */
    private function writeRecord():void {
        var xml2Write:XML = convertObject2XML();
        
        var file:File = File.desktopDirectory.resolvePath(User.XML_DEFAULT_FILE_NAME);

        if(writeXML(file, xml2Write)) {
            Alert.show(User.XML_DEFAULT_FILE_NAME + MessageConsts.MESSAGE_SUFFIX_FILE_WRITING_SUCCESS);
        } else {
            Alert.show(User.XML_DEFAULT_FILE_NAME + MessageConsts.MESSAGE_SUFFIX_FILE_WRITING_ERROR);
        }

        // ビジーカーソルを削除する
        CursorManager.removeBusyCursor();
    }
    
    /**
     * 実際のXML書き出し処理です。
     * 
     * @param file 書き出し先のファイルオブジェクトです。
     * @param xml 書き出すXMLオブジェクトです。
     * @return 書き出しに成功したかどうかのフラグです。
     */
    private function writeXML(file:File, xml:XML):Boolean {
        var stream:FileStream = new FileStream();
        var returnValue:Boolean;
        
        try {
            stream.open(file, FileMode.WRITE);
            stream.writeUTFBytes(xml.toString());
            returnValue = true;
        } catch(error:IOError) {
            returnValue = false;
        } finally {
            stream.close();
        }
        
        return returnValue;
    }

//--------------------------------------
//  Functions
//--------------------------------------
    
    /**
     * オブジェクトデータをXMLに変換します。
     */
    private function convertObject2XML():XML {
        var index:uint;
        var indexLimit:uint = selectedUserList.length;
        var tempXMLString:String;
        var tempUserString:String;
        
        tempXMLString = User.ROOT_START_TAG.concat(MessageConsts.BREAK);
        
        for(index = 0; index < indexLimit; index++) {
            tempUserString = User(selectedUserList.getItemAt(index)).toXMLString();
            
            // traceが記述されているとパフォーマンスが大幅に下がるので要らなくなったら削除しましょう。
            // 普通はロギングのフレームワークを作って対応します。
            //trace("convertObject2XML()", index, tempUserString);
            
            tempXMLString = tempXMLString.concat(tempUserString).concat(MessageConsts.BREAK);
        }

        tempXMLString = tempXMLString.concat(User.ROOT_END_TAG);
        
        return new XML(tempXMLString);
    }
    
    /**
     * 行番号ラベルファンクションです。
     * 
     * @param object 選択行のデータです。
     * @param column AdvancedDataGridColumnオブジェクトです。
     * @return 行番号です。
     */
    public function rowNumLabelFunction(object:Object, column:AdvancedDataGridColumn):String {
        var rowIndex:int = userList.getItemIndex(object) + 1;
        return String(rowIndex);
    }
  
    /**
     * 誕生日ラベルファンクションです。
     * 
     * @param object 選択行のデータです。
     * @param column AdvancedDataGridColumnです。
     * @return 誕生日です。
     */
    public function birthdayLabelFunction(object:Object, column:AdvancedDataGridColumn):String {
        var strings:Array = User(object).birthday.split("/");
        
        if(String(strings[1]).length == 1) {
            strings[1] = "0" + strings[1]
        }
        
        if(String(strings[2]).length == 1) {
            strings[2] = "0" + strings[2]
        }
        
        return strings[0] + "/" + strings[1] + "/" + strings[2];
    }
  
    /**
     * 性別フィルターファンクションです。
     * 
     * @param event ListEventオブジェクトです。
     */
    public function sexChangeHandler(event:ListEvent):void {
        if(userList != null) {
            if(view.sexCombo.selectedItem.value == "全性別") {
                selectedSex = "";
            } else {
                selectedSex = view.sexCombo.selectedItem.value;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();

            // 表示件数更新
            updateHitNum();
        }
    }

    /**
     * 血液型フィルターファンクションです。
     * 
     * @param event ListEventオブジェクトです。
     */
    public function bloodChangeHandler(event:ListEvent):void {
        if(userList != null) {
            if(view.bloodCombo.selectedItem.value == "全血液型") {
                selectedBloodType = "";
            } else {
                selectedBloodType = view.bloodCombo.selectedItem.value;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();

            // 表示件数更新
            updateHitNum();
        }
    }

    /**
     * 都道府県フィルターファンクションです。
     * 
     * @param event ListEventオブジェクトです。
     */
    public function preChangeHandler(event:ListEvent):void {
        if(userList != null) {
            if(view.preCombo.selectedItem.value == "全都道府県") {
                selectedPre = "";
            } else {
                selectedPre = view.preCombo.selectedItem.value;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();

            // 表示件数更新
            updateHitNum();
        }
    }
    
    /**
     * Eメールフィルターファンクションです。
     * 
     * @param event Eventオブジェクトです。
     */
    public function emailChangeHandler(event:Event):void {
        if(userList != null) {
            if(view.emailText.text == "") {
                selectedEmail = "";
            } else {
                selectedEmail = view.emailText.text;
            }
            
            // フィルターファンクション
            userList.filterFunction = doFilter;
            
            // フィルター処理を実行
            userList.refresh();
            
            // 表示件数更新
            updateHitNum();
        }
    }

    /**
     * メインフィルター処理です。
     * 
     * @param フィルター対象のオブジェクト1行1行です。
     * @return 行がフィルター条件に合うか合わないかです。
     */
    private function doFilter(value:Object):Boolean {
        var sexCheck:Boolean;
        var bloodTypeCheck:Boolean;
        var preCheck:Boolean;
        var emailCheck:Boolean;
        
        if(selectedSex == "" || selectedSex == null) {
            sexCheck = true;
        } else {
            sexCheck = (User(value).sex == selectedSex);
        }

        if(selectedBloodType == "" || selectedBloodType == null) {
            bloodTypeCheck = true;
        } else {
            bloodTypeCheck = (User(value).bloodtype == selectedBloodType);
        }

        if(selectedPre == "" || selectedPre == null) {
            preCheck = true;
        } else {
            preCheck = (User(value).prefecture == selectedPre);
        }
        
        if(selectedEmail == "" || selectedEmail == null) {
            emailCheck = true;
        } else {
            emailCheck = (User(value).mail.indexOf(selectedEmail) == 0);
        }

        return (sexCheck && bloodTypeCheck && preCheck && emailCheck);
    }
    
    /**
     * 表示件数更新です。
     */
    private function updateHitNum():void {
        if(userList != null) {
            hitNum = "（" + userList.length + "/" + userList.source.length + "）";
        }
    }
        
//--------------------------------------
//  View-Logic Binding
//--------------------------------------
    /** 画面です。 */
    public var _view:UserForm;
    
    /**
     * 画面を取得します。
     * 
     * @return 画面です。
     */     
    public function get view():UserForm {
        if (_view == null) {
            _view = super.document as UserForm;
        }
        return _view;
    }
    
    /**
     * 画面をセットします。
     * 
     * @param view セットする画面です。
     */
    public function set view(view:UserForm):void {
        _view = view;
    }
}
}